from .Logger import *
import requests
#import requests[socks] #youll need to pip install requests[socks] (this is only if youre using socks)
import json

class Login:
    '''
    Manages HTTP authentication
    '''
    URL = "https://discordapp.com/api/v8/auth/login" #self.URL

    def __init__(self, user_email, user_password,proxy_host,proxy_port):
        self.__user_email = user_email
        self.__user_password = user_password
        self.__proxy_host = proxy_host
        self.__proxy_port = proxy_port
        self.__token = None

    def Connect(self):
        session = requests.Session()
        if self.__proxy_host != False and self.__proxy_port != False:
            proxies = {
            'http': self.__proxy_host+':'+self.__proxy_port,
            'https': self.__proxy_host+':'+self.__proxy_port
            }
            session.proxies.update(proxies)
        session.headers.update({"User-Agent": "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) discord/0.0.305 Chrome/69.0.3497.128 Electron/4.0.8 Safari/537.36"})
        session.headers.update({'X-Super-Properties': ''})
        session.headers.update({"Content-Type": "application/json"})
        http_auth_data = '{{"email": "{}", "password": "{}", "undelete": false, "captcha_key": null, "login_source": null, "gift_code_sku_id": null}}'.format(self.__user_email, self.__user_password)
        Logger.LogMessage('Post -> {}'.format(self.URL))
        Logger.LogMessage('{}'.format(http_auth_data))
        response = session.post(self.URL, data=http_auth_data)
        Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
        self.__token = json.loads(response.content)['token']

    def GetToken(self):
        if self.__token is None:
            self.Connect()
        return self.__token



#!/usr/bin/env python
#-*-Mode:python;coding:utf-8;tab-width:4;c-basic-offset:4;indent-tabs-mode:()-*-
# ex: set ft=python fenc=utf-8 sts=4 ts=4 sw=4 et nomod:
#
# MIT License
#
# Copyright (c) 2011-2019 Michael Truog <mjtruog at protonmail dot com>
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
#
"""
Erlang Binary Term Format Encoding/Decoding
"""

import sys
import struct
import zlib
import copy

def etfDictToJson(target_dictionary, etf_data):
    if etf_data is None or type(etf_data) is not dict:
        return
    for key in etf_data.keys():
        if type(key) == str:
            index = key
        else:
            index = key.value
        serialized_val = etf_data[key]
        if type(serialized_val) == dict:
            target_dictionary[index] = {}
            etfDictToJson(target_dictionary[index], serialized_val)
            continue
        if type(serialized_val) == list:
            target_dictionary[index] = []
            for serialized_list_entry in serialized_val:
                if type(serialized_list_entry) == dict:
                    target_dictionary[index].append({})
                    etfDictToJson(target_dictionary[index][-1], serialized_list_entry)
                if type(serialized_list_entry) == OtpErlangBinary:
                    target_dictionary[index].append(serialized_list_entry.value)
                else:
                    target_dictionary[index].append(serialized_list_entry)
            continue
        if type(serialized_val) in (OtpErlangAtom, OtpErlangBinary):
            target_dictionary[index] = serialized_val.value
        else:
            target_dictionary[index] = serialized_val

def getName(erlangKey):
    erlangKeyText = "%s" % erlangKey
    if type(erlangKey) == OtpErlangAtom:
        searchText = "OtpErlangAtom('"
        if erlangKeyText.find(searchText) != -1:
            return (erlangKeyText)[erlangKeyText.find(searchText)+len(searchText):len(erlangKeyText)-2]
    elif type(erlangKey) == OtpErlangBinary:
        searchText1 = "OtpErlangBinary('"
        searchText2 = "',bits="
        if erlangKeyText.find(searchText1) != -1:
            return (erlangKeyText)[erlangKeyText.find(searchText1)+len(searchText1):erlangKeyText.find(searchText2)]
    return erlangKey

if sys.version_info[0] >= 3:
    TypeLong = int
    TypeUnicode = str
    def b_chr(integer):
        """
        bytes chr function
        """
        return bytes([integer])
    def b_ord(character):
        """
        bytes ord function
        """
        return character
else:
    TypeLong = long
    TypeUnicode = unicode
    def b_chr(integer):
        """
        bytes chr function
        """
        return chr(integer)
    def b_ord(character):
        """
        bytes ord function
        """
        return ord(character)

__all__ = ['OtpErlangAtom',
           'OtpErlangBinary',
           'OtpErlangFunction',
           'OtpErlangList',
           'OtpErlangPid',
           'OtpErlangPort',
           'OtpErlangReference',
           'binary_to_term',
           'term_to_binary',
           'InputException',
           'OutputException',
           'ParseException']

# tag values here http://www.erlang.org/doc/apps/erts/erl_ext_dist.html
_TAG_VERSION = 131
_TAG_COMPRESSED_ZLIB = 80
_TAG_NEW_FLOAT_EXT = 70
_TAG_BIT_BINARY_EXT = 77
_TAG_ATOM_CACHE_REF = 78
_TAG_NEW_PID_EXT = 88
_TAG_NEW_PORT_EXT = 89
_TAG_NEWER_REFERENCE_EXT = 90
_TAG_SMALL_INTEGER_EXT = 97
_TAG_INTEGER_EXT = 98
_TAG_FLOAT_EXT = 99
_TAG_ATOM_EXT = 100
_TAG_REFERENCE_EXT = 101
_TAG_PORT_EXT = 102
_TAG_PID_EXT = 103
_TAG_SMALL_TUPLE_EXT = 104
_TAG_LARGE_TUPLE_EXT = 105
_TAG_NIL_EXT = 106
_TAG_STRING_EXT = 107
_TAG_LIST_EXT = 108
_TAG_BINARY_EXT = 109
_TAG_SMALL_BIG_EXT = 110
_TAG_LARGE_BIG_EXT = 111
_TAG_NEW_FUN_EXT = 112
_TAG_EXPORT_EXT = 113
_TAG_NEW_REFERENCE_EXT = 114
_TAG_SMALL_ATOM_EXT = 115
_TAG_MAP_EXT = 116
_TAG_FUN_EXT = 117
_TAG_ATOM_UTF8_EXT = 118
_TAG_SMALL_ATOM_UTF8_EXT = 119

# Erlang term classes listed alphabetically

class OtpErlangAtom(object):
    """
    OtpErlangAtom
    """
    # pylint: disable=too-few-public-methods
    def __init__(self, value):
        self.value = value
    def binary(self):
        """
        return encoded representation
        """
        if isinstance(self.value, int):
            return b_chr(_TAG_ATOM_CACHE_REF) + b_chr(self.value)
        elif isinstance(self.value, TypeUnicode):
            value_encoded = self.value.encode('utf-8')
            length = len(value_encoded)
            if length <= 255:
                return (
                    b_chr(_TAG_SMALL_ATOM_UTF8_EXT) +
                    b_chr(length) + value_encoded
                )
            elif length <= 65535:
                return (
                    b_chr(_TAG_ATOM_UTF8_EXT) +
                    struct.pack(b'>H', length) + value_encoded
                )
            else:
                raise OutputException('uint16 overflow')
        elif isinstance(self.value, bytes):
            length = len(self.value)
            if length <= 255:
                return b_chr(_TAG_SMALL_ATOM_EXT) + b_chr(length) + self.value
            elif length <= 65535:
                return (
                    b_chr(_TAG_ATOM_EXT) +
                    struct.pack(b'>H', length) + self.value
                )
            else:
                raise OutputException('uint16 overflow')
        else:
            raise OutputException('unknown atom type')
    def __repr__(self):
        return '%s(%s)' % (self.__class__.__name__, repr(self.value))
    def __hash__(self):
        return hash(self.binary())
    def __eq__(self, other):
        return self.binary() == other.binary()

class OtpErlangBinary(object):
    """
    OtpErlangBinary
    """
    # pylint: disable=too-few-public-methods
    def __init__(self, value, bits=8):
        self.value = value
        self.bits = bits # bits in last byte
    def binary(self):
        """
        return encoded representation
        """
        if isinstance(self.value, bytes):
            length = len(self.value)
            if length > 4294967295:
                raise OutputException('uint32 overflow')
            elif self.bits != 8:
                return (
                    b_chr(_TAG_BIT_BINARY_EXT) +
                    struct.pack(b'>I', length) +
                    b_chr(self.bits) + self.value
                )
            else:
                return (
                    b_chr(_TAG_BINARY_EXT) +
                    struct.pack(b'>I', length) +
                    self.value
                )
        else:
            raise OutputException('unknown binary type')
    def __repr__(self):
        return '%s(%s,bits=%s)' % (
            self.__class__.__name__, repr(self.value), repr(self.bits)
        )
    def __hash__(self):
        return hash(self.binary())
    def __eq__(self, other):
        return self.binary() == other.binary()

class OtpErlangFunction(object):
    """
    OtpErlangFunction
    """
    # pylint: disable=too-few-public-methods
    def __init__(self, tag, value):
        self.tag = tag
        self.value = value
    def binary(self):
        """
        return encoded representation
        """
        return b_chr(self.tag) + self.value
    def __repr__(self):
        return '%s(%s,%s)' % (
            self.__class__.__name__,
            repr(self.tag), repr(self.value)
        )
    def __hash__(self):
        return hash(self.binary())
    def __eq__(self, other):
        return self.binary() == other.binary()

class OtpErlangList(object):
    """
    OtpErlangList
    """
    # pylint: disable=too-few-public-methods
    def __init__(self, value, improper=False):
        self.value = value
        self.improper = improper # no empty list tail?
    def binary(self):
        """
        return encoded representation
        """
        if isinstance(self.value, list):
            length = len(self.value)
            if length == 0:
                return b_chr(_TAG_NIL_EXT)
            elif length > 4294967295:
                raise OutputException('uint32 overflow')
            elif self.improper:
                return (
                    b_chr(_TAG_LIST_EXT) +
                    struct.pack(b'>I', length - 1) +
                    b''.join([_term_to_binary(element)
                              for element in self.value])
                )
            else:
                return (
                    b_chr(_TAG_LIST_EXT) +
                    struct.pack(b'>I', length) +
                    b''.join([_term_to_binary(element)
                              for element in self.value]) +
                    b_chr(_TAG_NIL_EXT)
                )
        else:
            raise OutputException('unknown list type')
    def __repr__(self):
        return '%s(%s,improper=%s)' % (
            self.__class__.__name__, repr(self.value), repr(self.improper)
        )
    def __hash__(self):
        return hash(self.binary())
    def __eq__(self, other):
        return self.binary() == other.binary()

class OtpErlangPid(object):
    """
    OtpErlangPid
    """
    # pylint: disable=too-few-public-methods
    def __init__(self, node, id_value, serial, creation):
        # pylint: disable=invalid-name
        self.node = node
        self.id = id_value
        self.serial = serial
        self.creation = creation
    def binary(self):
        """
        return encoded representation
        """
        creation_size = len(self.creation)
        if creation_size == 1:
            return (
                b_chr(_TAG_PID_EXT) +
                self.node.binary() + self.id + self.serial + self.creation
            )
        elif creation_size == 4:
            return (
                b_chr(_TAG_NEW_PID_EXT) +
                self.node.binary() + self.id + self.serial + self.creation
            )
        else:
            raise OutputException('unknown pid type')
    def __repr__(self):
        return '%s(%s,%s,%s,%s)' % (
            self.__class__.__name__,
            repr(self.node), repr(self.id), repr(self.serial),
            repr(self.creation)
        )
    def __hash__(self):
        return hash(self.binary())
    def __eq__(self, other):
        return self.binary() == other.binary()

class OtpErlangPort(object):
    """
    OtpErlangPort
    """
    # pylint: disable=too-few-public-methods
    def __init__(self, node, id_value, creation):
        # pylint: disable=invalid-name
        self.node = node
        self.id = id_value
        self.creation = creation
    def binary(self):
        """
        return encoded representation
        """
        creation_size = len(self.creation)
        if creation_size == 1:
            return (
                b_chr(_TAG_PORT_EXT) +
                self.node.binary() + self.id + self.creation
            )
        elif creation_size == 4:
            return (
                b_chr(_TAG_NEW_PORT_EXT) +
                self.node.binary() + self.id + self.creation
            )
        else:
            raise OutputException('unknown port type')
    def __repr__(self):
        return '%s(%s,%s,%s)' % (
            self.__class__.__name__,
            repr(self.node), repr(self.id), repr(self.creation)
        )
    def __hash__(self):
        return hash(self.binary())
    def __eq__(self, other):
        return self.binary() == other.binary()

class OtpErlangReference(object):
    """
    OtpErlangReference
    """
    # pylint: disable=too-few-public-methods
    def __init__(self, node, id_value, creation):
        # pylint: disable=invalid-name
        self.node = node
        self.id = id_value
        self.creation = creation
    def binary(self):
        """
        return encoded representation
        """
        length = int(len(self.id) / 4)
        if length == 0:
            return (
                b_chr(_TAG_REFERENCE_EXT) +
                self.node.binary() + self.id + self.creation
            )
        elif length <= 65535:
            creation_size = len(self.creation)
            if creation_size == 1:
                return (
                    b_chr(_TAG_NEW_REFERENCE_EXT) +
                    struct.pack(b'>H', length) +
                    self.node.binary() + self.creation + self.id
                )
            elif creation_size == 4:
                return (
                    b_chr(_TAG_NEWER_REFERENCE_EXT) +
                    struct.pack(b'>H', length) +
                    self.node.binary() + self.creation + self.id
                )
            else:
                raise OutputException('unknown reference type')
        else:
            raise OutputException('uint16 overflow')
    def __repr__(self):
        return '%s(%s,%s,%s)' % (
            self.__class__.__name__,
            repr(self.node), repr(self.id), repr(self.creation)
        )
    def __hash__(self):
        return hash(self.binary())
    def __eq__(self, other):
        return self.binary() == other.binary()

# dependency to support Erlang maps as map keys in python

class frozendict(dict):
    """
    frozendict is under the PSF (Python Software Foundation) License
    (from http://code.activestate.com/recipes/414283-frozen-dictionaries/)
    """
    # pylint: disable=invalid-name
    def _blocked_attribute(self):
        # pylint: disable=no-self-use
        raise AttributeError('A frozendict cannot be modified.')
    _blocked_attribute = property(_blocked_attribute)
    __delitem__ = __setitem__ = clear = _blocked_attribute
    pop = popitem = setdefault = update = _blocked_attribute
    def __new__(cls, *args, **kw):
        # pylint: disable=unused-argument
        # pylint: disable=too-many-nested-blocks
        new = dict.__new__(cls)
        args_ = []
        for arg in args:
            if isinstance(arg, dict):
                arg = copy.copy(arg)
                for k, v in arg.items():
                    if isinstance(v, dict):
                        arg[k] = frozendict(v)
                    elif isinstance(v, list):
                        v_ = list()
                        for elm in v:
                            if isinstance(elm, dict):
                                v_.append(frozendict(elm))
                            else:
                                v_.append(elm)
                        arg[k] = tuple(v_)
                args_.append(arg)
            else:
                args_.append(arg)
        dict.__init__(new, *args_, **kw)
        return new
    def __init__(self, *args, **kw):
        # pylint: disable=unused-argument
        # pylint: disable=super-init-not-called
        self.__cached_hash = None
    def __hash__(self):
        if self.__cached_hash is None:
            self.__cached_hash = hash(frozenset(self.items()))
        return self.__cached_hash
    def __repr__(self):
        return "frozendict(%s)" % dict.__repr__(self)

# core functionality

def binary_to_term(data):
    """
    Decode Erlang terms within binary data into Python types
    """
    if not isinstance(data, bytes):
        raise ParseException('not bytes input')
    size = len(data)
    if size <= 1:
        raise ParseException('null input')
    if b_ord(data[0]) != _TAG_VERSION:
        raise ParseException('invalid version')
    try:
        i, term = _binary_to_term(1, data)
        if i != size:
            raise ParseException('unparsed data')
        return term
    except struct.error:
        raise ParseException('missing data')
    except IndexError:
        raise ParseException('missing data')

def term_to_binary(term, compressed=False):
    """
    Encode Python types into Erlang terms in binary data
    """
    data_uncompressed = _term_to_binary(term)
    if compressed is False:
        return b_chr(_TAG_VERSION) + data_uncompressed
    else:
        if compressed is True:
            compressed = 6
        if compressed < 0 or compressed > 9:
            raise InputException('compressed in [0..9]')
        data_compressed = zlib.compress(data_uncompressed, compressed)
        size_uncompressed = len(data_uncompressed)
        if size_uncompressed > 4294967295:
            raise OutputException('uint32 overflow')
        return (
            b_chr(_TAG_VERSION) + b_chr(_TAG_COMPRESSED_ZLIB) +
            struct.pack(b'>I', size_uncompressed) + data_compressed
        )

# binary_to_term implementation functions

def _binary_to_term(i, data):
    # pylint: disable=too-many-locals
    # pylint: disable=too-many-return-statements
    # pylint: disable=too-many-branches
    # pylint: disable=too-many-statements
    tag = b_ord(data[i])
    i += 1
    if tag == _TAG_NEW_FLOAT_EXT:
        return (i + 8, struct.unpack(b'>d', data[i:i + 8])[0])
    elif tag == _TAG_BIT_BINARY_EXT:
        j = struct.unpack(b'>I', data[i:i + 4])[0]
        i += 4
        bits = b_ord(data[i])
        i += 1
        return (i + j, OtpErlangBinary(data[i:i + j], bits))
    elif tag == _TAG_ATOM_CACHE_REF:
        return (i + 1, OtpErlangAtom(b_ord(data[i])))
    elif tag == _TAG_SMALL_INTEGER_EXT:
        return (i + 1, b_ord(data[i]))
    elif tag == _TAG_INTEGER_EXT:
        return (i + 4, struct.unpack(b'>i', data[i:i + 4])[0])
    elif tag == _TAG_FLOAT_EXT:
        value = float(data[i:i + 31].partition(b_chr(0))[0])
        return (i + 31, value)
    elif tag == _TAG_ATOM_EXT:
        j = struct.unpack(b'>H', data[i:i + 2])[0]
        i += 2
        return (i + j, OtpErlangAtom(data[i:i + j]))
    elif (tag == _TAG_NEW_PORT_EXT or
          tag == _TAG_REFERENCE_EXT or tag == _TAG_PORT_EXT):
        i, node = _binary_to_atom(i, data)
        id_value = data[i:i + 4]
        i += 4
        if tag == _TAG_NEW_PORT_EXT:
            creation = data[i:i + 4]
            i += 4
        else:
            creation = data[i:i + 1]
            i += 1
            if tag == _TAG_REFERENCE_EXT:
                return (i, OtpErlangReference(node, id_value, creation))
        # tag == _TAG_NEW_PORT_EXT or tag == _TAG_PORT_EXT
        return (i, OtpErlangPort(node, id_value, creation))
    elif tag == _TAG_NEW_PID_EXT or tag == _TAG_PID_EXT:
        i, node = _binary_to_atom(i, data)
        id_value = data[i:i + 4]
        i += 4
        serial = data[i:i + 4]
        i += 4
        if tag == _TAG_NEW_PID_EXT:
            creation = data[i:i + 4]
            i += 4
        elif tag == _TAG_PID_EXT:
            creation = data[i:i + 1]
            i += 1
        return (i, OtpErlangPid(node, id_value, serial, creation))
    elif tag == _TAG_SMALL_TUPLE_EXT or tag == _TAG_LARGE_TUPLE_EXT:
        if tag == _TAG_SMALL_TUPLE_EXT:
            length = b_ord(data[i])
            i += 1
        elif tag == _TAG_LARGE_TUPLE_EXT:
            length = struct.unpack(b'>I', data[i:i + 4])[0]
            i += 4
        i, tuple_value = _binary_to_term_sequence(i, length, data)
        return (i, tuple(tuple_value))
    elif tag == _TAG_NIL_EXT:
        return (i, [])
    elif tag == _TAG_STRING_EXT:
        j = struct.unpack(b'>H', data[i:i + 2])[0]
        i += 2
        return (i + j, data[i:i + j])
    elif tag == _TAG_LIST_EXT:
        length = struct.unpack(b'>I', data[i:i + 4])[0]
        i += 4
        i, list_value = _binary_to_term_sequence(i, length, data)
        i, tail = _binary_to_term(i, data)
        if not isinstance(tail, list) or tail != []:
            list_value.append(tail)
            list_value = OtpErlangList(list_value, improper=True)
        return (i, list_value)
    elif tag == _TAG_BINARY_EXT:
        j = struct.unpack(b'>I', data[i:i + 4])[0]
        i += 4
        return (i + j, OtpErlangBinary(data[i:i + j], 8))
    elif tag == _TAG_SMALL_BIG_EXT or tag == _TAG_LARGE_BIG_EXT:
        if tag == _TAG_SMALL_BIG_EXT:
            j = b_ord(data[i])
            i += 1
        elif tag == _TAG_LARGE_BIG_EXT:
            j = struct.unpack(b'>I', data[i:i + 4])[0]
            i += 4
        sign = b_ord(data[i])
        bignum = 0
        for bignum_index in range(j):
            digit = b_ord(data[i + j - bignum_index])
            bignum = bignum * 256 + int(digit)
        if sign == 1:
            bignum *= -1
        i += 1
        return (i + j, bignum)
    elif tag == _TAG_NEW_FUN_EXT:
        length = struct.unpack(b'>I', data[i:i + 4])[0]
        return (i + length, OtpErlangFunction(tag, data[i:i + length]))
    elif tag == _TAG_EXPORT_EXT:
        old_i = i
        i, _ = _binary_to_atom(i, data)
        i, _ = _binary_to_atom(i, data)
        if b_ord(data[i]) != _TAG_SMALL_INTEGER_EXT:
            raise ParseException('invalid small integer tag')
        i += 1
        _ = b_ord(data[i])
        i += 1
        return (i, OtpErlangFunction(tag, data[old_i:i]))
    elif tag == _TAG_NEWER_REFERENCE_EXT or tag == _TAG_NEW_REFERENCE_EXT:
        j = struct.unpack(b'>H', data[i:i + 2])[0] * 4
        i += 2
        i, node = _binary_to_atom(i, data)
        if tag == _TAG_NEWER_REFERENCE_EXT:
            creation = data[i:i + 4]
            i += 4
        elif tag == _TAG_NEW_REFERENCE_EXT:
            creation = data[i:i + 1]
            i += 1
        return (i + j, OtpErlangReference(node, data[i: i + j], creation))
    elif tag == _TAG_SMALL_ATOM_EXT:
        j = b_ord(data[i])
        i += 1
        atom_name = data[i:i + j]
        i = i + j
        if atom_name == b'true':
            return (i, True)
        elif atom_name == b'false':
            return (i, False)
        return (i, OtpErlangAtom(atom_name))
    elif tag == _TAG_MAP_EXT:
        length = struct.unpack(b'>I', data[i:i + 4])[0]
        i += 4
        pairs = {}
        for _ in range(length):
            i, key = _binary_to_term(i, data)
            i, value = _binary_to_term(i, data)
            if isinstance(key, dict):
                pairs[frozendict(key)] = value
            elif isinstance(key, list):
                pairs[OtpErlangList(key)] = value
            else:
                pairs[key] = value
        return (i, pairs)
    elif tag == _TAG_FUN_EXT:
        old_i = i
        numfree = struct.unpack(b'>I', data[i:i + 4])[0]
        i += 4
        i, _ = _binary_to_pid(i, data)
        i, _ = _binary_to_atom(i, data)
        i, _ = _binary_to_integer(i, data)
        i, _ = _binary_to_integer(i, data)
        i, _ = _binary_to_term_sequence(i, numfree, data)
        return (i, OtpErlangFunction(tag, data[old_i:i]))
    elif tag == _TAG_ATOM_UTF8_EXT:
        j = struct.unpack(b'>H', data[i:i + 2])[0]
        i += 2
        atom_name = TypeUnicode(
            data[i:i + j], encoding='utf-8', errors='strict'
        )
        return (i + j, OtpErlangAtom(atom_name))
    elif tag == _TAG_SMALL_ATOM_UTF8_EXT:
        j = b_ord(data[i])
        i += 1
        atom_name = TypeUnicode(
            data[i:i + j], encoding='utf-8', errors='strict'
        )
        return (i + j, OtpErlangAtom(atom_name))
    elif tag == _TAG_COMPRESSED_ZLIB:
        size_uncompressed = struct.unpack(b'>I', data[i:i + 4])[0]
        if size_uncompressed == 0:
            raise ParseException('compressed data null')
        i += 4
        data_compressed = data[i:]
        j = len(data_compressed)
        data_uncompressed = zlib.decompress(data_compressed)
        if size_uncompressed != len(data_uncompressed):
            raise ParseException('compression corrupt')
        (i_new, term) = _binary_to_term(0, data_uncompressed)
        if i_new != size_uncompressed:
            raise ParseException('unparsed data')
        return (i + j, term)
    else:
        raise ParseException('invalid tag')

def _binary_to_term_sequence(i, length, data):
    sequence = []
    for _ in range(length):
        i, element = _binary_to_term(i, data)
        sequence.append(element)
    return (i, sequence)

# (binary_to_term Erlang term primitive type functions)

def _binary_to_integer(i, data):
    tag = b_ord(data[i])
    i += 1
    if tag == _TAG_SMALL_INTEGER_EXT:
        return (i + 1, b_ord(data[i]))
    elif tag == _TAG_INTEGER_EXT:
        return (i + 4, struct.unpack(b'>i', data[i:i + 4])[0])
    else:
        raise ParseException('invalid integer tag')

def _binary_to_pid(i, data):
    tag = b_ord(data[i])
    i += 1
    if tag == _TAG_NEW_PID_EXT:
        i, node = _binary_to_atom(i, data)
        id_value = data[i:i + 4]
        i += 4
        serial = data[i:i + 4]
        i += 4
        creation = data[i:i + 4]
        i += 4
        return (i, OtpErlangPid(node, id_value, serial, creation))
    elif tag == _TAG_PID_EXT:
        i, node = _binary_to_atom(i, data)
        id_value = data[i:i + 4]
        i += 4
        serial = data[i:i + 4]
        i += 4
        creation = data[i:i + 1]
        i += 1
        return (i, OtpErlangPid(node, id_value, serial, creation))
    else:
        raise ParseException('invalid pid tag')

def _binary_to_atom(i, data):
    tag = b_ord(data[i])
    i += 1
    if tag == _TAG_ATOM_EXT:
        j = struct.unpack(b'>H', data[i:i + 2])[0]
        i += 2
        return (i + j, OtpErlangAtom(data[i:i + j]))
    elif tag == _TAG_ATOM_CACHE_REF:
        return (i + 1, OtpErlangAtom(b_ord(data[i])))
    elif tag == _TAG_SMALL_ATOM_EXT:
        j = b_ord(data[i])
        i += 1
        return (i + j, OtpErlangAtom(data[i:i + j]))
    elif tag == _TAG_ATOM_UTF8_EXT:
        j = struct.unpack(b'>H', data[i:i + 2])[0]
        i += 2
        atom_name = TypeUnicode(
            data[i:i + j], encoding='utf-8', errors='strict'
        )
        return (i + j, OtpErlangAtom(atom_name))
    elif tag == _TAG_SMALL_ATOM_UTF8_EXT:
        j = b_ord(data[i])
        i += 1
        atom_name = TypeUnicode(
            data[i:i + j], encoding='utf-8', errors='strict'
        )
        return (i + j, OtpErlangAtom(atom_name))
    else:
        raise ParseException('invalid atom tag')

# term_to_binary implementation functions

def _term_to_binary(term):
    # pylint: disable=too-many-return-statements
    # pylint: disable=too-many-branches
    if isinstance(term, bytes):
        return _string_to_binary(term)
    elif isinstance(term, TypeUnicode):
        return _string_to_binary(
            term.encode(encoding='utf-8', errors='strict')
        )
    elif isinstance(term, list):
        return OtpErlangList(term).binary()
    elif isinstance(term, tuple):
        return _tuple_to_binary(term)
    elif isinstance(term, bool):
        return OtpErlangAtom(term and b'true' or b'false').binary()
    elif isinstance(term, (int, TypeLong)):
        return _long_to_binary(term)
    elif isinstance(term, float):
        return _float_to_binary(term)
    elif isinstance(term, dict):
        return _dict_to_binary(term)
    elif term is None:
        return OtpErlangAtom(b'undefined').binary()
    elif isinstance(term, OtpErlangAtom):
        return term.binary()
    elif isinstance(term, OtpErlangList):
        return term.binary()
    elif isinstance(term, OtpErlangBinary):
        return term.binary()
    elif isinstance(term, OtpErlangFunction):
        return term.binary()
    elif isinstance(term, OtpErlangReference):
        return term.binary()
    elif isinstance(term, OtpErlangPort):
        return term.binary()
    elif isinstance(term, OtpErlangPid):
        return term.binary()
    else:
        raise OutputException('unknown python type')

# (term_to_binary Erlang term composite type functions)

def _string_to_binary(term):
    length = len(term)
    if length == 0:
        return b_chr(_TAG_NIL_EXT)
    elif length <= 65535:
        return b_chr(_TAG_STRING_EXT) + struct.pack(b'>H', length) + term
    elif length <= 4294967295:
        return (
            b_chr(_TAG_LIST_EXT) + struct.pack(b'>I', length) +
            b''.join([b_chr(_TAG_SMALL_INTEGER_EXT) + b_chr(b_ord(c))
                      for c in term]) +
            b_chr(_TAG_NIL_EXT)
        )
    else:
        raise OutputException('uint32 overflow')

def _tuple_to_binary(term):
    length = len(term)
    if length <= 255:
        return (
            b_chr(_TAG_SMALL_TUPLE_EXT) + b_chr(length) +
            b''.join([_term_to_binary(element) for element in term])
        )
    elif length <= 4294967295:
        return (
            b_chr(_TAG_LARGE_TUPLE_EXT) + struct.pack(b'>I', length) +
            b''.join([_term_to_binary(element) for element in term])
        )
    else:
        raise OutputException('uint32 overflow')

def _dict_to_binary(term):
    length = len(term)
    if length <= 4294967295:
        return (
            b_chr(_TAG_MAP_EXT) + struct.pack(b'>I', length) +
            b''.join([_term_to_binary(key) + _term_to_binary(value)
                      for key, value in term.items()])
        )
    else:
        raise OutputException('uint32 overflow')

# (term_to_binary Erlang term primitive type functions)

def _integer_to_binary(term):
    if 0 <= term <= 255:
        return b_chr(_TAG_SMALL_INTEGER_EXT) + b_chr(term)
    return b_chr(_TAG_INTEGER_EXT) + struct.pack(b'>i', term)

def _long_to_binary(term):
    if -2147483648 <= term <= 2147483647:
        return _integer_to_binary(term)
    return _bignum_to_binary(term)

def _bignum_to_binary(term):
    bignum = abs(term)
    if term < 0:
        sign = b_chr(1)
    else:
        sign = b_chr(0)
    value = []
    while bignum > 0:
        value.append(b_chr(bignum & 255))
        bignum >>= 8
    length = len(value)
    if length <= 255:
        return (
            b_chr(_TAG_SMALL_BIG_EXT) +
            b_chr(length) + sign + b''.join(value)
        )
    elif length <= 4294967295:
        return (
            b_chr(_TAG_LARGE_BIG_EXT) +
            struct.pack(b'>I', length) + sign + b''.join(value)
        )
    else:
        raise OutputException('uint32 overflow')

def _float_to_binary(term):
    return b_chr(_TAG_NEW_FLOAT_EXT) + struct.pack(b'>d', term)

# Exception classes listed alphabetically

class InputException(ValueError):
    """
    InputError describes problems with function input parameters
    """
    def __init__(self, s):
        ValueError.__init__(self)
        self.__s = str(s)
    def __str__(self):
        return self.__s

class OutputException(TypeError):
    """
    OutputError describes problems with creating function output data
    """
    def __init__(self, s):
        TypeError.__init__(self)
        self.__s = str(s)
    def __str__(self):
        return self.__s

class ParseException(SyntaxError):
    """
    ParseError provides specific parsing failure information
    """
    def __init__(self, s):
        SyntaxError.__init__(self)
        self.__s = str(s)
    def __str__(self):
        return self.__s

def consult(string_in):
    """
    provide file:consult/1 functionality with python types
    """
    # pylint: disable=eval-used
    # pylint: disable=too-many-branches
    # pylint: disable=too-many-statements

    # manually parse textual erlang data to avoid external dependencies
    list_out = []
    tuple_binary = False   # binaries become tuples of integers
    quoted_string = False  # strings become python string
    atom_string = False    # atoms become python string
    number = False
    whitespace = frozenset(('\n', '\t', ' '))
    i = 0
    while i < len(string_in):
        character = string_in[i]
        if character == ',':
            if atom_string:
                list_out.append('"')
                atom_string = False
            list_out.append(',')
            number = string_in[i + 1].isdigit()
        elif character == '{':
            list_out.append('(')
            number = string_in[i + 1].isdigit()
        elif character == '}':
            if atom_string:
                list_out.append('"')
                atom_string = False
            list_out.append(')')
            number = False
        elif character == '[':
            list_out.append('[')
            number = string_in[i + 1].isdigit()
        elif character == ']':
            if atom_string:
                list_out.append('"')
                atom_string = False
            list_out.append(']')
            number = False
        elif character == '<' and string_in[i + 1] == '<':
            list_out.append('(')
            tuple_binary = True
            i += 1
        elif character == '>' and string_in[i + 1] == '>':
            list_out.append(')')
            tuple_binary = False
            i += 1
        elif not quoted_string and not atom_string and character in whitespace:
            number = string_in[i + 1].isdigit()
        elif tuple_binary or number:
            list_out.append(character)
        elif character == '"':
            if quoted_string:
                quoted_string = False
            else:
                quoted_string = True
            list_out.append('"')
        elif character == "'":
            if atom_string:
                atom_string = False
            else:
                atom_string = True
            list_out.append('"')
        elif not quoted_string and not atom_string:
            atom_string = True
            list_out.append('"')
            list_out.append(character)
        else:
            list_out.append(character)
        i += 1
    return eval(''.join(list_out))



import websocket
import ssl
from .erlang import *
from .erlang import etfDictToJson,getName #just in case
from .Logger import *

class GatewayServer():
    '''
    Manages authentication and data transfer to/from http://gateway.discord.gg
    '''
    URL = "wss://gateway.discord.gg/?encoding=etf&v=6"

    class OPCODE:
        '''
        Taken from https://discordapp.com/developers/docs/topics/opcodes-and-status-codes#voice-opcodes
        '''
        # Name                  Code    Client Action   Description
        DISPATCH =              0  #    Receive         dispatches an event
        HEARTBEAT =             1  #    Send/Receive    used for ping checking
        IDENTIFY =              2  #    Send            used for client handshake
        STATUS =                3  #    Send            used to update the client status
        VOICE_STATE =           4  #    Send            used to join/move/leave voice channels
        #                       5  #    ???             ???
        RESUME =                6  #    Send            used to resume a closed connection
        RECONNECT =             7  #    Receive         used to tell clients to reconnect to the gateway
        REQUEST_GUILD_MEMBERS = 8  #    Send            used to request guild members
        INVALID_SESSION =       9  #    Receive         used to notify client they have an invalid session id
        HELLO =                 10 #    Receive         sent immediately after connecting, contains heartbeat and server debug information
        HEARTBEAT_ACK =         11 #    Sent immediately following a client heartbeat that was received
        GUILD_SYNC =            12 #    ???             ???

    def __init__(self, proxy_host, proxy_port):
        self.__proxy_host = proxy_host
        self.__proxy_port = proxy_port
        self.__ws = websocket.WebSocket(sslopt={"check_hostname": False, "cert_reqs": ssl.CERT_NONE})
        self.__session_settings = {}
        self.__ws.settimeout(5)
        
    def Connect(self, token):
        gateway_auth_data = {OtpErlangBinary(str.encode('op'),bits=8): 2,
                OtpErlangBinary(str.encode('d'),bits=8):
                    {OtpErlangBinary(str.encode('properties'),bits=8):
                        {OtpErlangBinary(str.encode('client_version'),bits=8): OtpErlangBinary(str.encode('0.0.305'),bits=8),
                        OtpErlangBinary(str.encode('os'),bits=8): OtpErlangBinary(str.encode('Windows'),bits=8),
                        OtpErlangBinary(str.encode('os_version'),bits=8): OtpErlangBinary(str.encode('6.1.7601'),bits=8),
                        OtpErlangBinary(str.encode('os_arch'),bits=8): OtpErlangBinary(str.encode('x64'),bits=8),
                        OtpErlangBinary(str.encode('release_channel'),bits=8): OtpErlangBinary(str.encode('stable'),bits=8),
                        OtpErlangBinary(str.encode('client_build_number'),bits=8): 44004,
                        OtpErlangBinary(str.encode('browser'),bits=8): OtpErlangBinary(str.encode('Discord Client'),bits=8),
                        OtpErlangBinary(str.encode('client_event_source'),bits=8): OtpErlangAtom('nil')
                        },
                    OtpErlangBinary(str.encode('compress'),bits=8): False,
                    OtpErlangBinary(str.encode('token'),bits=8): OtpErlangBinary(str.encode("{}".format(token)), bits=8),
                    OtpErlangBinary(str.encode('presence'),bits=8):
                        {OtpErlangBinary(str.encode('status'),bits=8): OtpErlangBinary(str.encode('online'),bits=8),
                        OtpErlangBinary(str.encode('since'),bits=8): 0,
                        OtpErlangBinary(str.encode('afk'),bits=8): False,
                        OtpErlangBinary(str.encode('activities'),bits=8): []
                        }
                    }
                }
        # Connect to gateway
        gateway_auth_data = term_to_binary(gateway_auth_data)
        if self.__proxy_host != False and self.__proxy_port != False:
            self.__ws.connect(self.URL, origin="https://discordapp.com", http_proxy_host=self.__proxy_host, http_proxy_port=self.__proxy_port, header={"User-Agent": "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) discord/0.0.305 Chrome/69.0.3497.128 Electron/4.0.8 Safari/537.36"})
        else:
            self.__ws.connect(self.URL, origin="https://discordapp.com", header={"User-Agent": "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) discord/0.0.305 Chrome/69.0.3497.128 Electron/4.0.8 Safari/537.36"})
        Logger.LogMessage('Gateway_wss connecting -> {}'.format(self.URL), log_level=LogLevel.OK)

        # Receive response
        response = str(self.__ws.recv())
        Logger.LogMessage('Gateway_wss received <- {} bytes'.format(len(response)))
        Logger.LogMessage('Gateway_wss data received \r\n', to_file=False, to_console=False, hex_data=response)

        # Send auth message
        Logger.LogMessage('Gateway_wss sending -> {} bytes'.format(len(gateway_auth_data)))
        self.__ws.send_binary(gateway_auth_data)
        Logger.LogMessage('Gateway_wss data sent -> \r\n', to_file=False, to_console=False, hex_data=gateway_auth_data)

        # Receive media_token
        response = self.__ws.recv()
        etf_data = binary_to_term(response)
        Logger.LogMessage('Gateway_wss received <- {} bytes'.format(len(response)))
        Logger.LogMessage('Gateway_wss data received <- \r\n', to_file=False, to_console=False, hex_data=response)
        etfDictToJson(self.__session_settings, etf_data[OtpErlangAtom(str.encode('d'))]) #self.__session_settings has a lot of important data, not to mention free server.members yay
        Logger.LogMessage('Authenticated over gateway as {}'.format(self.__session_settings[str.encode('user')][str.encode('id')]), log_level=LogLevel.OK)
        Logger.LogMessage('Session id: {}'.format(self.__session_settings[str.encode('session_id')].decode("utf-8")))
        
    def return_session_settings():
        return self.__session_settings



import inspect

class LogLevel:
    INFO = '\033[94m'
    OK = '\033[92m'
    WARNING = '\033[93m'
    DEFAULT = '\033[m'

class Logger:
    @staticmethod
    def LogMessage(msg, hex_data='', to_file=False, to_console=True, log_level=LogLevel.INFO): #to_file was acting a bit buggy so I decided to remove it altogether for now
        stack = inspect.stack()
        function_name = "({}->{})".format(str(stack[1][0].f_locals['self']).split(' ')[0], stack[1][3])
        if to_console is True:
            if hex_data is not '':
                print('{} {}'.format(log_level, " ".join([h.encode('hex') for h in hex_data])))
            else:
                print('{} [+] {} {}'.format(log_level, function_name, msg))
            print(LogLevel.DEFAULT) # restore console color



from .messages.messages import Messages
from .messages.embed import Embedder
from .user.user import User

from .Login import *
from .GatewayServer import *

import time
import random

class Settings:
	def __init__(self, obj):
		for k, v in obj.items():
			if isinstance(v, dict):
				setattr(self, k, Settings(v))
			else:
				setattr(self, k, v)
	def __getitem__(self, val):
		return self.__dict__[val]
	def __repr__(self):
		return '{%s}' % str(', '.join('%s : %s' % (k, repr(v)) for (k, v) in self.__dict__.items()))

class Client:
    def convert(self, data):
    	if isinstance(data, bytes):
    		return data.decode()
    	if isinstance(data, dict):
    		return dict(map(self.convert, data.items()))
    	if isinstance(data, tuple):
    		return tuple(map(self.convert, data))
    	if isinstance(data, list):
    		return list(map(self.convert, data))
    	return data
    def __init__(self, email="none", password="none", token="none", proxy_host=False, proxy_port=False): #not using None since that could get flagged by discord...
        self.__user_token = token
        self.__user_email = email
        self.__user_password = password
        self.__proxy_host = proxy_host
        self.__proxy_port = proxy_port
        self.classsession_settings = {} #look at function read()
        if self.__user_token == "none": #assuming email and pass are given...
        	self.__login = Login(self.__user_email, self.__user_password,self.__proxy_host,self.__proxy_port)
        	self.__user_token = self.__login.GetToken() #update token from "none" to true string value
        self.__gateway_server = GatewayServer(self.__proxy_host,self.__proxy_port)
        self.discord = 'https://discord.com/api/v8/'
        self.headers = {
        "Host": "discord.com",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) discord/0.0.306 Chrome/78.0.3904.130 Electron/7.1.11 Safari/537.36",
        "Accept": "*/*",
        "Accept-Language": "en-US",
        "Authorization": self.__user_token,
        "Connection": "keep-alive",
        "keep-alive" : "timeout=10, max=1000",
        "TE": "Trailers",
        "Pragma": "no-cache",
        "Cache-Control": "no-cache",
        "Referer": "https://discord.com/channels/@me",
        "Content-Type": "application/json"
        }
        self.s = requests.Session()
        self.s.headers.update(self.headers)
        if self.__proxy_host != False and self.__proxy_port != False: #self.s.proxies defaults to {}
            self.proxies = {
            'http': self.__proxy_host+':'+self.__proxy_port,
            'https': self.__proxy_host+':'+self.__proxy_port
            }
            self.s.proxies.update(proxies)

    '''
    discord snowflake to unix timestamp and back

    '''
    def snowflake_to_unixts(self,snowflake):
        return int((snowflake/4194304+1420070400000)/1000)

    def unixts_to_snowflake(self,unixts):
        return int((unixts*1000-1420070400000)*4194304)

    '''
    (get) and/or read session settings/data
    '''
    def read(self,update=True): #returns a class, this is the main function, if you want ALL the session data (wall of data), then call this (or bot.read().__dict__). if update=False session_settings will not be updated
    	if update == False: #if read() hasnt been called yet this will just return an empty dict
    		return self.classsession_settings
    	self.__gateway_server.Connect(self.__user_token)
    	session_settings = self._Client__gateway_server._GatewayServer__session_settings
    	if isinstance(session_settings[b'_trace'],bytes): #if bytes shows up.......................idk sometimes it shows up sometimes it doesnt so this is just in case
    		session_settings[b'_trace'] = session_settings[b'_trace'][0].decode("utf-8")
    	strsession_settings = self.convert(session_settings) #this is a dict, the str denotes that there are no byte keys, only string keys
    	self.classsession_settings = Settings(strsession_settings)
    	return self.classsession_settings

    def getAnalyticsToken(self,update=True):
    	return self.read(update).analytics_token

    def getConnectedAccounts(self,update=True):
    	return self.read(update).connected_accounts

    def getConsents(self,update=True):
    	return self.read(update).consents

    def getExperiments(self,update=True):
    	return self.read(update).experiments

    def getFriendSuggestingCount(self,update=True): #no idea what this is but it's here so whatever
    	return self.read(update).friend_suggestion_count

    def getGuildExperiments(self,update=True):
    	return self.read(update).guild_experiments


    #All about guilds, oh geez this is a long one
    def getGuilds(self,update=True): #returns all information about all guilds you're in...might be a little overwhelming so don't call unless you're ready to see a wall of text
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	return [self.read(False).guilds[i] for i in range(len(self.read(False).guilds)) if i%2==0] #remove the erlang trash, which are on odd indices

    def getGuildIDs(self,update=True): #just get the guild ids, type list
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	return [self.getGuilds(False)[i]['id'] for i in range(len(self.getGuilds(False)))]


    ## getting specific about a PARTICULAR guild, note that guildID is an int, but if you happen to pass a string that's fine...I know most libs out there pass the guildID as a string so you might be used to doing that
    def getGuildData(self,guildID,update=True): #type dict, all data about a PARTICULAR guild, can be overwhelming, guildID is an int, but if you happen to pass a string that's fine
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	for i in range(len(self.getGuilds(False))):
    		if self.getGuilds(False)[i]['id'] == guildID:
    			return self.getGuilds(False)[i]
    	return None #guild not found

    def getGuildOwner(self,guildID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['owner_id'] #returns type int

    def getGuildBoostLvl(self,guildID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['premium_tier']

    def getGuildEmojis(self,guildID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['emojis']

    def getGuildBanner(self,guildID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['banner'] #if returns 'nil' then there's no banner

    def getGuildDiscoverySplash(self,guildID,update=True): #not sure what this is about, something about server discoverability i guess (https://discord.com/developers/docs/resources/guild)
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['discovery_splash']

    def getGuildUserPresences(self,guildID,update=True): #only returns presences of online, idle, or do-not-disturb users
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return [self.getGuildData(guildID,False)['presences'][i] for i in range(len(self.getGuildData(guildID,False)['presences'])) if i%2==0]

    def getGuildMsgNotificationSettings(self,guildID,update=True): #returns an int, 0=all messages, 1=only mentions (https://discord.com/developers/docs/resources/guild#guild-object-default-message-notification-level)
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['default_message_notifications']

    def getGuildRulesChannelID(self,guildID,update=True): #nil if no rules channel id, idk if this always works so it might actually be more useful just to look for the word "rules" in channel names
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['rules_channel_id']

    def getGuildVerificationLvl(self,guildID,update=True): #returns an int, 0-4 (https://discord.com/developers/docs/resources/guild#guild-object-verification-level)
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['verification_level']

    def getGuildFeatures(self,guildID,update=True): #returns a list of strings (https://discord.com/developers/docs/resources/guild#guild-object-guild-features)
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['features']

    def getGuildJoinTime(self,guildID,update=True): #returns when you joined the server, type string
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['joined_at']

    def getGuildRegion(self,guildID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['region']

    def getGuildApplicationID(self,GuildID,update=True): #returns application id of the guild creator if it is bot-created (https://discord.com/developers/docs/resources/guild#guild-object-guild-features)
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['application_id']

    def getGuildAfkChannelID(self,guildID,update=True): #not sure what this is
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['afk_channel_id']

    def getGuildIcon(self,guildID,update=True): #https://discord.com/developers/docs/reference#image-formatting
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['icon']

    def getGuildName(self,guildID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['name']

    def getGuildMaxVideoChannelUsers(self,guildID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['max_video_channel_users']

    def getGuildRoles(self,guildID,update=True): #https://discord.com/developers/docs/topics/permissions#role-object
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return [self.getGuildData(guildID,False)['roles'][i] for i in range(len(self.getGuildData(guildID,False)['roles'])) if i%2==0]

    def getGuildPublicUpdatesChannelID(self,guildID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['public_updates_channel_id']

    def getGuildSystemChannelFlags(self,guildID,update=True): #https://discord.com/developers/docs/resources/guild#guild-object-system-channel-flags
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['system_channel_flags']

    def getGuildMfaLvl(self,guildID,update=True): #https://discord.com/developers/docs/resources/guild#guild-object-mfa-level
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['mfa_level']

    def getGuildAfkTimeout(self,guildID,update=True): #returns type int, unit seconds, https://discord.com/developers/docs/resources/guild
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['afk_timeout']

    def getGuildHashes(self,guildID,update=True): #https://github.com/discord/discord-api-docs/issues/1642
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['guild_hashes']

    def getGuildSystemChannelID(self,guildID,update=True): #returns an int, the id of the channel where guild notices such as welcome messages and boost events are posted
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['system_channel_id']

    def isGuildLazy(self,guildID,update=True): #slightly different naming format since it returns a boolean (https://luna.gitlab.io/discord-unofficial-docs/lazy_guilds.html)
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	if self.getGuildData(guildID,False)['lazy'] == 'true':
    		return True
    	else:
    		return False

    def getGuildNumBoosts(self,guildID,update=True): #get number of boosts the server has gotten
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['premium_subscription_count']

    def isGuildLarge(self,guildID,update=True): #slightly different naming format since it returns a boolean, large if more than 250 members
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	if self.getGuildData(guildID,False)['large'] == 'true':
    		return True
    	else:
    		return False

    def getGuildExplicitContentFilter(self,guildID,update=True): #https://discord.com/developers/docs/resources/guild#guild-object-explicit-content-filter-level
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['explicit_content_filter']

    def getGuildSplashHash(self,guildID,update=True): #not sure what this is for
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['splash']

    def getGuildVoiceStates(self,guildID,update=True): #https://discord.com/developers/docs/resources/voice#voice-state-object
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return [self.getGuildData(guildID,False)['voice_states'][i] for i in range(len(self.getGuildData(guildID,False)['voice_states'])) if i%2==0]

    def getGuildMemberCount(self,guildID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['member_count']

    def getGuildDescription(self,guildID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['description']

    def getGuildVanityUrlCode(self,guildID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['vanity_url_code']

    def getGuildPreferredLocale(self,guildID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return self.getGuildData(guildID,False)['preferred_locale']

    def getGuildAllChannels(self,guildID,update=True): #returns all categories and all channels, all the data about that, wall of data so it can be a bit overwhelming, useful if you want to check how many channels your server has since discord counts categories as channels
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return [self.getGuildData(guildID,False)['channels'][i] for i in range(len(self.getGuildData(guildID,False)['channels'])) if i%2==0]

    def getGuildCategories(self,guildID,update=True): #all data about all guild categories, can be overwhelming
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	all_channels = [self.getGuildData(guildID,False)['channels'][i] for i in range(len(self.getGuildData(guildID,False)['channels'])) if i%2==0]
    	all_categories = []
    	for channel in all_channels: #https://discord.com/developers/docs/resources/channel#channel-object-channel-types
    		if channel['type'] == 4:
    			all_categories.append(channel)
    	return all_categories

    def getGuildCategoryIDs(self,guildID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return [self.getGuildCategories(guildID,False)[i]['id'] for i in range(len(self.getGuildCategories(guildID,False)))]

    def getGuildCategoryData(self,guildID,categoryID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	if isinstance(categoryID,str):
    		categoryID = int(categoryID)
    	for i in range(len(self.getGuildCategories(guildID,False))):
    		if self.getGuildCategories(guildID,False)[i]['id'] == categoryID:
    			return self.getGuildCategories(guildID,False)[i]
    	return None #category not found

    def getGuildChannels(self,guildID,update=True): #all data about all guild channels, can be overwhelming
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	all_channels = [self.getGuildData(guildID,False)['channels'][i] for i in range(len(self.getGuildData(guildID,False)['channels'])) if i%2==0]
    	all_non_categories = []
    	for channel in all_channels: #https://discord.com/developers/docs/resources/channel#channel-object-channel-types
    		if channel['type'] != 4:
    			all_non_categories.append(channel)
    	return all_non_categories

    def getGuildChannelIDs(self,guildID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return [self.getGuildChannels(guildID,False)[i]['id'] for i in range(len(self.getGuildChannels(guildID,False)))]

    def getGuildChannelData(self,guildID,channelID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	if isinstance(channelID,str):
    		channelID = int(channelID)
    	for i in range(len(self.getGuildChannels(guildID,False))):
    		if self.getGuildChannels(guildID,False)[i]['id'] == channelID:
    			return self.getGuildChannels(guildID,False)[i]
    	return None #channel not found

    def getGuildMembers(self,guildID,update=True): #all data about all members, can be overwhelming....doesnt get all members for some reason...my guess is that it only gets most recently active members but idk
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return [self.getGuildData(guildID,False)['members'][i] for i in range(len(self.getGuildData(guildID,False)['members'])) if i%2==0]

    def getGuildMemberIDs(self,guildID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	return [self.getGuildMembers(guildID,False)[i]['user']['id'] for i in range(len(self.getGuildMembers(guildID,False)))]

    def getGuildMemberData(self,guildID,memberID,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(guildID,str):
    		guildID = int(guildID)
    	if isinstance(memberID,str):
    		memberID = int(memberID)
    	for i in range(len(self.getGuildMembers(guildID,False))):
    		if self.getGuildMembers(guildID,False)[i]['user']['id'] == memberID:
    			return self.getGuildMembers(guildID,False)[i]
    	return None #member not found, again I must state that getGuildMembers doesnt return all members, likely it only returns most recent members but who knows for now...

    # end of guild stuff, now onto the other stuff

    def getNotes(self,update=True):
    	return self.read(update).notes #returns a dict where the keys are user IDs and the values are the notes

    def getOnlineFriends(self,update=True): #i saw no reason to make "sub functions" of this since it's only about online friends
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	[self.read(False).presences[i] for i in range(len(self.read(False).presences)) if i%2==0]

    #all about DMs, aw geez this is gonna be a long one too i guess...
    def getDMs(self,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	return [self.read(False).private_channels[i] for i in range(len(self.read(False).private_channels)) if i%2==0]

    def getDMIDs(self,update=True): #discord sometimes calls these channel IDs...
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	return [self.getDMs(False)[i]['id'] for i in range(len(self.getDMs(False)))]

    def getDMData(self,DMID,update=True): #get data about a particular DM
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(DMID,str):
    		DMID = int(DMID)
    	for i in range(len(self.getDMs(False))):
    		if self.getDMs(False)[i]['id'] == DMID:
    			return self.getDMs(False)[i]
    	return None

    def getDMRecipients(self,DMID,update=True): #returns everyone in that DM except your user
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	if isinstance(DMID,str):
    		DMID = int(DMID)
    	return [self.getDMData(DMID,False)['recipients'][i] for i in range(len(self.getDMData(DMID,False)['recipients'])) if i%2==0]

    #end of DM stuff, heh that wasnt that bad, onto the rest of the stuff

    def getReadStates(self,update=True): #another advantage of using websockets instead of requests (see https://github.com/discord/discord-api-docs/issues/13)
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	return [self.read(False).read_state[i] for i in range(len(self.read(False).read_state)) if i%2==0]

    #all about relationships...on discord. note this includes all your friends AND all users youve blocked
    def getRelationships(self,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	return [self.read(False).relationships[i] for i in range(len(self.read(False).relationships)) if i%2==0]

    def getRelationshipIDs(self,update=True): #gets userIDs that are in a "relationship" with your account
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	return [self.getRelationships(False)[i]['id'] for i in range(len(self.getRelationships(False)))]

    def getRelationshipData(self,RelationshipID,update=True):
        getthatdata = self.read(update) #refreshes session settings if update is True
        if isinstance(RelationshipID,str) and "#" in RelationshipID: #if in username discriminator format
            for i in range(len(self.getRelationships(False))):
                if self.getRelationships(False)[i]['user']['username'] == RelationshipID.split("#")[0] and self.getRelationships(False)[i]['user']['discriminator'] == RelationshipID.split("#")[1]:
                    return self.getRelationships(False)[i]
        elif isinstance(RelationshipID,str): #if in ID format, but is a string instead of an int
            RelationshipID = int(RelationshipID)
        for i in range(len(self.getRelationships(False))):
            if self.getRelationships(False)[i]['id'] == RelationshipID:
                return self.getRelationships(False)[i]
        return None

    def getFriends(self,update=True): #yay, no this will not give you friends, it returns data about the users you have a friends
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	friends = []
    	for i in range(len(self.getRelationships(False))):
    		if self.getRelationships(False)[i]['type'] == 1:
    			friends.append(self.getRelationships(False)[i])
    	return friends #wow and just like that you have friends.........................

    def getFriendIDs(self,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	return [self.getFriends(False)[i]['id'] for i in range(len(self.getFriends(False)))]

    def getBlocked(self,update=True): #nay
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	blocked = []
    	for i in range(len(self.getRelationships(False))):
    		if self.getRelationships(False)[i]['type'] == 2:
    			blocked.append(self.getRelationships(False)[i])
    	return blocked

    def getBlockedIDs(self,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	return [self.getBlocked(False)[i]['id'] for i in range(len(self.getBlocked(False)))]

    def getIncomingFriendRequests(self,update=True): #returns data...
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	ifr = []
    	for i in range(len(self.getRelationships(False))):
    		if self.getRelationships(False)[i]['type'] == 3:
    			ifr.append(self.getRelationships(False)[i])
    	return ifr

    def getIncomingFriendRequestIDs(self,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	return [self.getIncomingFriendRequests(False)[i]['id'] for i in range(len(self.getIncomingFriendRequests(False)))]

    def getOutgoingFriendRequests(self,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	ofr = []
    	for i in range(len(self.getRelationships(False))):
    		if self.getRelationships(False)[i]['type'] == 4:
    			ofr.append(self.getRelationships(False)[i])
    	return ofr

    def getOutgoingFriendRequestIDs(self,update=True):
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	return [self.getOutgoingFriendRequests(False)[i]['id'] for i in range(len(self.getOutgoingFriendRequests(False)))]

    #end of relationship stuff

    def getSessionID(self,update=True):
    	return self.read(update).session_id

    def getTutorial(self,update=True): #tutorial on what? guess we'll never know...ask discord maybe?
    	return self.read(update).tutorial

    def getUserData(self,update=True):
    	return self.read(update).user

    def getUserGuildSettings(self,update=True,guildID=None): #personal settings for a server, like whether or not you want @everyone pings to show, formatting is weird cause you might not want to pass a guildID, hence it is at the end
    	getthatdata = self.read(update) #refreshes session settings if update is True
    	all_user_guild_settings = [self.read(False).user_guild_settings[i] for i in range(len(self.read(False).user_guild_settings)) if i%2==0]
    	if guildID != None:
    		if isinstance(guildID,str):
    			guildID = int(guildID)
    		for i in range(len(all_user_guild_settings)):
    			if all_user_guild_settings[i]['guild_id'] == guildID:
    				return all_user_guild_settings[i]
    	else:
    		return all_user_guild_settings

    def getUserSettings(self,update=True): #returns a class
    	return self.read(update).user_settings

    def getOptionsForUserSettings(self,update=True): #bc i didnt feel like making a ton of sub functions for this, the data is there if you want to process it but rn it seems useless to me
    	return list(self.read(update).user_settings.__dict__.keys()) #since getUserSettings() returns a class, you can call subsettings using this format: getUserSettings().afk_timeout

    def getWebsocketVersion(self,update=True):
    	return self.read(update).v

    # oof end of reading session settings


    '''
    username to snowflake and back
    '''
    def username_to_userID(self,userdiscriminator): #userdiscriminator is "username#discriminator"
        getthatdata = self.read() #might as well get current session settings
        if type(self.getRelationshipData(userdiscriminator,False)).__name__ != 'NoneType': #testing our luck to see if user is in a "relationship" with our user
            return self.getRelationshipData(userdiscriminator,False)['id'] #returns type int
        User(self.discord,self.s).requestFriend(userdiscriminator) #this puts you in a "relationship" with that user
        if type(self.getRelationshipData(userdiscriminator,True)).__name__ != 'NoneType': #if you sent a request to a bot or to yourself then this becomes False
            User(self.discord,self.s).removeRelationship(self.getRelationshipData(userdiscriminator,False)['id'])
            return self.getRelationshipData(userdiscriminator,False)['id'] #returns type int
        return None #happens if other user is a bot, is yourself, does not exist, or something wrong with your account

    def userID_to_username(self,userID): #userID aka snowflake
        getthatdata = self.read() #might as well get current session settings
        if type(self.getRelationshipData(snowflake,False)).__name__ != 'NoneType': #testing our luck to see if user is in a "relationship" with our user
            userdiscriminator = self.getRelationshipData(snowflake,False)['user']['username'] + "#" + self.getRelationshipData(snowflake,False)['user']['discriminator']
            return userdiscriminator
        if isinstance(snowflake,int):
            snowflake = str(snowflake)
        User(self.discord,self.s).requestFriend(snowflake) #this puts you in a "relationship" with that user
        if type(self.getRelationshipData(snowflake,True)).__name__ != 'NoneType': #if you sent a request to a bot or to yourself then this becomes False
            User(self.discord,self.s).removeRelationship(snowflake)
            userdiscriminator = self.getRelationshipData(snowflake,False)['user']['username'] + "#" + self.getRelationshipData(snowflake,False)['user']['discriminator']
            return userdiscriminator
        return None #happens if other user is a bot, is yourself, does not exist, or something wrong with your account


    '''
    Messages
    '''
    #get recent messages
    def getMessages(self,channelID,num=1,beforeDate=None): # num <= 100, beforeDate is a snowflake
        if isinstance(channelID,int):
            channelID = str(channelID)
        return Messages(self.discord,self.s).getRecentMessage(channelID,num,beforeDate)

    #send text or embed messages
    def sendMessage(self,channelID,message,embed="",tts=False):
        if isinstance(channelID,int):
            channelID = str(channelID)
        return Messages(self.discord,self.s).sendMessage(channelID,message,embed,tts)

    #send files (local or link)
    def sendFile(self,channelID,filelocation,isurl=False,message=""):
        if isinstance(channelID,int):
            channelID = str(channelID)
        return Messages(self.discord,self.s).sendFile(channelID,filelocation,isurl,message)

    #search messages
    def searchMessages(self,guildID,channelID=None,userID=None,mentionsUserID=None,has=None,beforeDate=None,afterDate=None,textSearch=None,afterNumResults=None):
        if isinstance(guildID,int):
            guildID = str(guildID)
        return Messages(self.discord,self.s).searchMessages(guildID,channelID,userID,mentionsUserID,has,beforeDate,afterDate,textSearch,afterNumResults)

    #filter searchMessages, takes in the output of searchMessages (a requests response object) and outputs a list of target messages
    def filterSearchResults(self,searchResponse):
        return Messages(self.discord,self.s).filterSearchResults(searchResponse)

    #sends the typing action for 10 seconds (or technically until you change the page)
    def typingAction(self,channelID):
        if isinstance(channelID,int):
            channelID = str(channelID)
        return Messages(self.discord,self.s).typingAction(channelID)

    #delete message
    def deleteMessage(self,channelID,messageID):
        if isinstance(channelID,int):
            channelID = str(channelID)
        if isinstance(messageID,int):
            messageID = str(messageID)
        return Messages(self.discord,self.s).deleteMessage(channelID,messageID)

    #pin message
    def pinMessage(self,channelID,messageID):
        if isinstance(channelID,int):
            channelID = str(channelID)
        if isinstance(messageID,int):
            messageID = str(messageID)
        return Messages(self.discord,self.s).pinMessage(channelID,messageID)

    #un-pin message
    def unPinMessage(self,channelID,messageID):
        if isinstance(channelID,int):
            channelID = str(channelID)
        if isinstance(messageID,int):
            messageID = str(messageID)
        return Messages(self.discord,self.s).unPinMessage(channelID,messageID)

    #get pinned messages
    def getPins(self,channelID):
        if isinstance(channelID,int):
            channelID = str(channelID)
        return Messages(self.discord,self.s).getPins(channelID)

    '''
    User relationships
    '''
    #create outgoing friend request
    def requestFriend(self,user): #you can input a userID(snowflake) or a user discriminator
        if isinstance(user,int): #only affects userIDs cause...well...strings...
            user = str(user)
        return User(self.discord,self.s).requestFriend(user)

    #accept incoming friend request
    def acceptFriend(self,userID):
        if isinstance(userID,int):
            userID = str(userID)
        return User(self.discord,self.s).acceptFriend(userID)

    #remove friend OR unblock user
    def removeRelationship(self,userID):
        if isinstance(userID,int):
            userID = str(userID)
        return User(self.discord,self.s).removeRelationship(userID)

    #block user
    def blockUser(self,userID):
        if isinstance(userID,int):
            userID = str(userID)
        return User(self.discord,self.s).blockUser(userID)

    '''
    Profile edits
    '''
    # change name
    def changeName(self,name):
        return User(self.discord,self.s).changeName(self.email,self.password,name)
    # set status
    def setStatus(self,status):
        return User(self.discord,self.s).setStatus(status)
    # set avatar
    def setAvatar(self,imagePath):
        return User(self.discord,self.s).setAvatar(self.email,self.password,imagePath)



import requests
import json
import base64
from ..Logger import *

class User(object):
	def __init__(self, discord, s): #s is the requests session object
		self.discord = discord
		self.s = s
		
	#def getDMs(self): #websockets does this now
	#	url = self.discord+"users/@me/channels"
	#	return self.s.get(url)

	#def getGuilds(self): #websockets does this now
	#	url = self.discord+"users/@me/guilds"
	#	return self.s.get(url)

	#def getRelationships(self): #websockets does this now
	#	url = self.discord+"users/@me/relationships"
	#	return self.s.get(url)

	def requestFriend(self,user):
		if "#" in user:
			url = self.discord+"users/@me/relationships"
			body = {"username": user.split("#")[0], "discriminator": int(user.split("#")[1])}
			Logger.LogMessage('Post -> {}'.format(url))
			Logger.LogMessage('{}'.format(str(body)))
			response = self.s.post(url, data=json.dumps(body))
			Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
			return response
		url = self.discord+"users/@me/relationships/"+user
		Logger.LogMessage('Put -> {}'.format(url))
		response = self.s.put(url, data=json.dumps({}))
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response

	def acceptFriend(self,userID):
		url = self.discord+"users/@me/relationships/"+userID
		Logger.LogMessage('Put -> {}'.format(url))
		response = self.s.put(url, data=json.dumps({}))
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response

	def removeRelationship(self,userID): #for removing friends, unblocking people
		url = self.discord+"users/@me/relationships/"+userID
		Logger.LogMessage('Delete -> {}'.format(url))
		response = self.s.delete(url)
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response

	def blockUser(self,userID):
		url = self.discord+"users/@me/relationships/"+userID
		Logger.LogMessage('Put -> {}'.format(url))
		Logger.LogMessage('{}'.format(str({"type":2})))
		response = self.s.put(url, data=json.dumps({"type":2}))
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response

	'''
	Profile Edits
	'''
	def changeName(self,email,password,name):
		url = self.discord+"users/@me"
		Logger.LogMessage('Patch -> {}'.format(url))
		Logger.LogMessage('{}'.format(str({"username":name,"email":email,"password":password})))
		response = self.s.patch(url, data=json.dumps({"username":name,"email":email,"password":password}))
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response
	
	def setStatus(self,status):
		url = self.discord+"users/@me/settings"
		Logger.LogMessage('Patch -> {}'.format(url))
		if(status == 0): # Online
			Logger.LogMessage('{}'.format(str({"status":"online"})))
			response = self.s.patch(url, data=json.dumps({"status":"online"}))
			Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
			return response
		elif(status == 1): # Idle
			Logger.LogMessage('{}'.format(str({"status":"idle"})))
			response = self.s.patch(url, data=json.dumps({"status":"idle"}))
			Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
			return response
		elif(status == 2): #Do Not Disturb
			Logger.LogMessage('{}'.format(str({"status":"dnd"})))
			response = self.s.patch(url, data=json.dumps({"status":"dnd"}))
			Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
			return response
		elif (status == 3): #Invisible
			Logger.LogMessage('{}'.format(str({"status":"invisible"})))
			response = self.s.patch(url, data=json.dumps({"status":"invisible"}))
			Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
			return response
		elif (status == ''):
			Logger.LogMessage('{}'.format(str({"custom_status":None})))
			response = self.s.patch(url, data=json.dumps({"custom_status":None}))
			Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
			return response
		else:
			Logger.LogMessage('{}'.format(str({"custom_status":{"text":status}})))
			response = self.s.patch(url, data=json.dumps({"custom_status":{"text":status}}))
			Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
			return response

	def setAvatar(self,email,password,imagePath): #local image path
		url = self.discord+"users/@me"
		Logger.LogMessage('Patch -> {}'.format(url))
		Logger.LogMessage('{}'.format(str({"email":email,"password":password,"avatar":"data:image/png;base64,<encoded image data>","discriminator":None,"new_password":None})))
		with open(imagePath, "rb") as image:
			encodedImage = base64.b64encode(image.read()).decode('utf-8')
		response = self.s.patch(url, data=json.dumps({"email":email,"password":password,"avatar":"data:image/png;base64,"+encodedImage,"discriminator":None,"new_password":None}))
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response
		



import requests
import json
from ..fileparse.fileparse import Fileparse
from ..Logger import *
from urllib.parse import urlparse,quote_plus
from requests_toolbelt import MultipartEncoder
import random,string
import math
import os
import time

class Messages(object):
	def __init__(self, discord, s): #s is the requests session object
		self.discord = discord
		self.s = s

	#get Message
	def getMessages(self,channelID,num,beforeDate): # 1 ≤ num ≤ 100, beforeDate is a snowflake
		url = self.discord+"channels/"+channelID+"/messages?limit="+str(num)
		if beforeDate != None:
			url += "&before="+str(beforeDate)
		Logger.LogMessage('Get -> {}'.format(url))
		response = self.s.get(url)
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response

	#text message
	def sendMessage(self,channelID,message,embed,tts):
		url = self.discord+"channels/"+channelID+"/messages"
		body = {"content": message, "tts": tts,"embed":embed}
		Logger.LogMessage('Post -> {}'.format(url))
		Logger.LogMessage('{}'.format(str(body)))
		response = self.s.post(url, data=json.dumps(body))
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response

	#send file
	def sendFile(self,channelID,filelocation,isurl,message):
		mimetype, extensiontype, fd = Fileparse(self.s).parse(filelocation,isurl) #guess extension from file data
		if mimetype == 'invalid': #error out
			print('ERROR: File does not exist.')
			return
		if isurl: #get filename
			a = urlparse(filelocation)
			if len(os.path.basename(a.path))>0: #if everything is normal...
				filename = os.path.basename(a.path)
			else: 
				if mimetype == 'unsupported': #if filetype not detected and extension not given
					filename = 'unknown'
				else: #if filetype detected but extension not given
					filename = 'unknown.'+extensiontype
		else: #local file
			filename = os.path.basename(os.path.normpath(filelocation))
		#now time to post the file
		url = self.discord+'channels/'+channelID+'/messages'
		if isurl:
			fields={"file":(filename,fd,mimetype),"file_id":"0", "content":message}
		else:
			fields={"file":(filename,open(filelocation,'rb').read(),mimetype),"file_id":"0", "content":message}
		m=MultipartEncoder(fields=fields,boundary='----WebKitFormBoundary'+''.join(random.sample(string.ascii_letters+string.digits,16)))
		self.s.headers.update({"Content-Type":m.content_type})
		Logger.LogMessage('Post -> {}'.format(url))
		Logger.LogMessage('{}'.format(str(MultipartEncoder(fields={"file":(filename,"<file data here>",mimetype),"file_id":"0", "content":message},boundary='----WebKitFormBoundary'+''.join(random.sample(string.ascii_letters+string.digits,16))))))
		response = self.s.post(url, data=m)
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response

	def searchMessages(self,guildID,channelID,userID,mentionsUserID,has,beforeDate,afterDate,textSearch,afterNumResults): #classic discord search function, results with key "hit" are the results you searched for, afterNumResults (aka offset) is multiples of 25 and indicates after which messages (type int), filterResults defaults to False
		url = self.discord+"guilds/"+guildID+"/messages/search?"
		queryparams = ""
		if not all(v is None for v in [channelID,userID,mentionsUserID,has,beforeDate,afterDate,textSearch,afterNumResults]):
			if channelID != None and isinstance(channelID,list):
				for item in channelID:
					if isinstance(item,int):
						queryparams += "channel_id="+str(item)+"&"
					elif isinstance(item,str) and len(item)>0:
						queryparams += "channel_id="+item+"&"
			if userID != None and isinstance(userID,list):
				for item in userID:
					if isinstance(item,int):
						queryparams += "author_id="+str(item)+"&"
					elif isinstance(item,str) and len(item)>0:
						queryparams += "author_id="+item+"&"
			if mentionsUserID != None and isinstance(mentionsUserID,list):
				for item in mentionsUserID:
					if isinstance(item,int):
						queryparams += "mentions="+str(item)+"&"
					elif isinstance(item,str) and len(item)>0:
						queryparams += "mentions="+item+"&"
			if has != None and isinstance(has,list):
				for item in has:
					if isinstance(item,str) and len(item)>0:
						queryparams += "has="+item+"&"
			if beforeDate != None and isinstance(beforeDate,int):
				queryparams += "min_id="+str(beforeDate)+"&"
			if afterDate != None and isinstance(afterDate,int):
				queryparams += "max_id="+str(afterDate)+"&"
			if textSearch != None and isinstance(textSearch,str): #textSearch can be len 0....ugh
				queryparams += "content="+quote_plus(textSearch)+"&"
			if afterNumResults != None and isinstance(afterNumResults,int):
				queryparams += "offset="+str(afterNumResults)
			url += queryparams
			Logger.LogMessage('Get -> {}'.format(url))
			response = self.s.get(url)
			Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
			return response
		Logger.LogMessage('Get -> {}'.format(url))
		response = self.s.get(url)
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response

	def filterSearchResults(self,searchResponse): #only input is the requests response object outputted from searchMessages, returns type list
		jsonresponse = searchResponse.json()['messages']
		filteredMessages = []
		for group in jsonresponse:
			for result in group:
				if 'hit' in result:
					filteredMessages.append(result)
		return filteredMessages

	def typingAction(self,channelID): #sends the typing action for 10 seconds (or until you change the page)
		url = self.discord+"channels/"+channelID+"/typing"
		Logger.LogMessage('Post -> {}'.format(url))
		response = self.s.post(url)
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response

	def editMessage(self,channelID,messageID,newMessage):
		url = self.discord+"channels/"+channelID+"/messages/"+messageID
		body = {"content": newMessage}
		Logger.LogMessage('Patch -> {}'.format(url))
		Logger.LogMessage('{}'.format(str(body)))
		response = self.s.patch(url, data=json.dumps(body))
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response

	def deleteMessage(self,channelID,messageID):
		url = self.discord+"channels/"+channelID+"/messages/"+messageID
		Logger.LogMessage('Delete -> {}'.format(url))
		response = self.s.delete(url)
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response

	def pinMessage(self,channelID,messageID):
		url = self.discord+"channels/"+channelID+"/pins/"+messageID
		Logger.LogMessage('Put -> {}'.format(url))
		response = self.s.put(url)
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response

	def unPinMessage(self,channelID,messageID):
		url = self.discord+"channels/"+channelID+"/pins/"+messageID
		Logger.LogMessage('Delete -> {}'.format(url))
		response = self.s.delete(url)
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response

	def getPins(self,channelID): #get pinned messages
		url = self.discord+"channels/"+channelID+"/pins"
		Logger.LogMessage('Get -> {}'.format(url))
		response = self.s.get(url)
		Logger.LogMessage('Response <- {}'.format(response.text), log_level=LogLevel.OK)
		return response
